﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace IgniteUISample
{
    /// <summary>
    ///     A wrapper class that provides oData paging support to lists.
    /// </summary>
    /// <typeparam name="T">The type of item stored in the list.</typeparam>
    public class ListResult<T>
    {
        //=========================================================================================
        //Nested Types
        //=========================================================================================
        #region Nested Types

        /// <summary>
        ///     Provides storage for the data contained within an instance of 
        ///     <see cref="ListResult"/>.
        /// </summary>
        /// <typeparam name="U">The type of item stored in the list.</typeparam>
        public class ListResultData<U>
        {
            /// <summary>
            ///     Gets or sets the data stored within this instance of <b>ListResultData</b>
            /// </summary>
            public List<U> results { get; set; }

            /// <summary>
            ///     Gets or sets the total count of items that would be returned if paging were not
            ///     enabled.
            /// </summary>
            public long __count { get; set; }
        }

        #endregion Nested Types

        //=========================================================================================
        //Constructors
        //=========================================================================================
        #region Constructors

        /// <summary>
        ///     Initializes a new instance of <strong>ListResult</strong>, using the passed data
        ///     and total count.
        /// </summary>
        /// <param name="items">The data to be wrapped in the list result.</param>
        /// <param name="total">
        ///     The total count of items that would be returned by a query, were paging not
        ///     enabled.
        /// </param>
        public ListResult(IEnumerable<T> items, long total)
        {
            d = new ListResultData<T>();
            d.results = items.ToList();
            d.__count = total;

        }

        #endregion Constructors

        //=========================================================================================
        //Properties
        //=========================================================================================
        #region Properties
        
        /// <summary>
        ///     Gets or sets the internal storage buffer for this instance of 
        ///     <strong>ListResult</strong>.
        /// </summary>
        public ListResultData<T> d { get; set; }

        #endregion Properties
        
    }
}