﻿(function ($) {

    var regexes = {
        queryRegex: new RegExp("([?].*)?$", "i")
    };

    var methods = {

        admedGrid: function () {

            return this.each(function () {

                var $table = $(this);

                var gridAttributes = {
                    del: $table.attr("data-grid-delete"),
                    filter: $table.attr("data-grid-filter"),
                    hiding: $table.attr("data-grid-hiding"),
                    add: $table.attr("data-grid-insert"),
                    paging: $table.attr("data-grid-paging"),
                    pagesize: $table.attr("data-grid-paging-size"),
                    primaryKey: $table.attr("data-grid-primarykey"),
                    rowRendered: $table.attr("data-grid-row-rendered"),
                    rowSelectors: $table.attr("data-grid-row-selectors"),
                    selection: $table.attr("data-grid-selection"),
                    selectionChanged: $table.attr("data-grid-selection-changed"),
                    selectionmode: $table.attr("data-grid-selection-mode"),
                    sortmode: $table.attr("data-grid-sorting"),
                    source: $table.attr("data-grid-source"),
                    updateUrl: $table.attr("data-grid-update"),
                    width: $table.attr("data-grid-width"),
                    refresh: $table.attr("data-grid-refresh"),
                    disablessort: $table.attr("data-grid-disable-sort"),
                    fixedHeaders: $table.attr("data-grid-fixedHeaders")
                };

                var features = {
                    filtering: { name: 'Filtering', type: gridAttributes.source ? 'remote' : 'local', mode: gridAttributes.filter, allowFiltering: true, columnSettings: [] },
                    sorting: { name: 'Sorting', type: gridAttributes.source ? 'remote' : 'local', modalDialogSortOnClick: true, applySortedColumnCss: false, mode: gridAttributes.sortmode ? gridAttributes.sortmode : "single", columnSettings: [] },
                    update: { name: 'Updating', editMode: gridAttributes.updateUrl ? "row" : "none", enableAddRow: gridAttributes.add ? gridAttributes.add : false, enableDeleteRow: gridAttributes.del ? gridAttributes.del : false, columnSettings: [] },
                    hiding: { name: 'Hiding', columnSettings: [] },
                    tooltips: { name: 'Tooltips', columnSettings: [] },
                    paging: { name: 'Paging', type: gridAttributes.source ? 'remote' : 'local', pageSize: gridAttributes.pagesize ? gridAttributes.pagesize : 100, pageSizeDropDownLocation: "inpager" },
                    rowSelectors: { name: "RowSelectors", enableCheckBoxes: true, enableRowNumbering: false, rowSelectorColumnWidth: 45, requireSelection: true, checkBoxStateChanged: window[gridAttributes.selectionChanged] },
                    selection: { name: 'Selection', multipleSelection: gridAttributes.rowSelectors ? true : false, activation: true, mode: gridAttributes.selectionmode ? gridAttributes.selectionmode : "row",
                        rowSelectionChanging: function (evt, ui) {
                            // This default behavior for the rowSelectionChanged event is used to prevent clicking outside of the checkboxes from deselecting all currently-selected rows
                            //      and to act the same as if the user had clicked inside the checkbox.
                            var gridId = ui.owner.element[0].id;
                            var row = $(ui.row.element);

                            if ($(ui.row.element).find("td.ui-iggrid-selectedcell").length == 0) {
                                setTimeout(function () {
                                    $("#" + gridId).igGridSelection("selectRow", ui.row.index);
                                    methods._rowSelectionChanged(evt, ui, gridAttributes);
                                }, 100);
                            } else {
                                setTimeout(function () {
                                    $("#" + gridId).igGridSelection("deselectRow", ui.row.index);
                                    methods._rowSelectionChanged(evt, ui, gridAttributes);
                                }, 100);
                            }
                            return false;
                        },
                        cellSelectionChanging: function (evt, ui) {
                            return false;
                        },
                        rowSelectionChanged: function (evt, ui, features) {
                            methods._rowSelectionChanged(evt, ui, gridAttributes);
                        }
                    }
                };

                var ajaxTooltips = [];
                var columnlinks = [];

                var schema = [];

                var gridSettings = {
                    alternateRowStyles: true,
                    autofitLastColumn: true,
                    autoGenerateColumns: false,
                    columns: [],
                    features: [],
                    virtualization: false,
                    primaryKey: gridAttributes.primaryKey,
                    renderCheckboxes: true,
                    rowsRendered: function (evt, ui) {
                        methods._renderTooltips(ui.owner, ajaxTooltips);
                        methods._renderLinks(ui.owner, columnlinks);
                        methods._renderRows(ui.owner, gridAttributes.rowRendered);
                        $.admed.parse(ui.owner.rows());
                        ui.owner.rows().find("td").css("vertical-align", "top");
                        ui.owner.rows().find("span[name=chk]").css("border", "solid 1px #696969");
                        ui.owner.element.find("span[name=hchk]").css("border", "solid 1px #696969");
                    },
                    dataRendered: function (evt, ui) {

                        var $g = $(ui.owner.element);

                        var ct = $g.find("tbody > tr").length;

                        var theadCt = $g.find("colgroup > col").length;

                        if (ct == 0) {
                            $g.find("tbody").html("<tr><td class='textColor' style='font-style:italic' colspan='" + theadCt + "'>No records</td></tr>");
                        }

                        if ($g.attr("data-grid-refresh")) {

                            $g.find("thead .refreshigGrid").closest("tr").remove();
                            var refreshId = "__refreshId_" + Math.floor(Math.random() * 1000);
                            $g.find("thead").prepend("<tr><th class='ui-iggrid-header ui-widget-header ui-state-default' colspan='" + theadCt + "'><div class='refreshigGrid'><span class=' ui-icon-btn-clearfilter'></span>Clear Filters</div></th></tr>");
                            $(".refreshigGrid").button();
                        }
                    }
                };

                gridSettings.features.push(features.tooltips);
                gridSettings.features.push(features.update);

                if (!gridAttributes.disablessort) gridSettings.features.push(features.sorting);
                if (gridAttributes.selection) gridSettings.features.push(features.selection);
                if (gridAttributes.width) gridSettings.width = gridAttributes.width;
                if (gridAttributes.filter) gridSettings.features.push(features.filtering);
                if (gridAttributes.hiding) gridSettings.features.push(features.hiding);
                if (gridAttributes.paging) gridSettings.features.push(features.paging);
                if (gridAttributes.rowSelectors) gridSettings.features.push(features.selection, features.rowSelectors);
                if (gridAttributes.updateUrl) gridSettings.updateUrl = gridAttributes.updateUrl;
                if (gridAttributes.fixedHeaders) gridSettings.fixedHeaders = "false";



                $table.find("th").each(function (index) {

                    var $column = $(this);

                    var columnAttributes = {
                        format: $column.attr("data-col-format"),
                        hidden: $column.attr("data-col-hidden"),
                        display: $column.attr("data-col-display") || "true",
                        key: $column.attr("data-col-key"),
                        required: $column.attr("data-col-required"),
                        tooltips: $column.attr("data-col-tooltip"),
                        tooltipUrl: $column.attr("data-col-tooltip-url"),
                        tooltipWidth: $column.attr("data-col-tooltip-width"),
                        tooltipColKey: $column.attr("data-col-tooltip-columnKey"),
                        dataType: $column.attr("data-col-type"),
                        showFalse: $column.attr("data-col-showfalse"),
                        width: $column.attr("data-col-width"),
                        update: $column.attr("data-col-update"),
                        editor: $column.attr("data-col-update-editor"),
                        editorUrl: $column.attr("data-col-update-editor-url"),
                        colUrl: $column.attr("data-col-url"),
                        isExternalLink: $column.attr("data-col-externallink"),
                        displayText: $column.attr("data-col-displaytext"),
                        isPopupWindow: $column.attr("data-col-popupWindow"),
                        popupWindowTitle: $column.attr("data-col-popupWindowTitle"),
                        disableSort: $column.attr("data-col-disableSort"),
                        disableFilter: $column.attr("data-col-disableFilter"),
                        template: $column.attr("data-col-template"),
                        unbound: $column.attr("data-col-unbound")
                    };


                    var formatter = null;
                    var headerText = $.trim($column.text());
                    var key = headerText;
                    var fieldRequired = false;
                    var hidden = columnAttributes.display === "false" ? true : false;
                    var sort = (columnAttributes.disableSort != null || columnAttributes.disableSort != undefined) ? false : true;
                    var filter = (columnAttributes.disableFilter != null || columnAttributes.disableFilter != undefined) ? false : true;

                    if (hidden) { sort = false; filter = false; }

                    if (columnAttributes.key) key = columnAttributes.key;
                    if (columnAttributes.required) fieldRequired = columnAttributes.required;
                    if (columnAttributes.hidden && columnAttributes.hidden == "true") hidden = true;

                    if (columnAttributes.dataType == "boolean") {
                        columnAttributes.template = '{{if ${' + columnAttributes.key + '} == "true"}}<span style="width:100%;display:inline-block;overflow:hidden;text-align:center;"><span class="ui-igcheckbox-small" style="display:inline-block"><span style="display:block" class="ui-icon ui-icon-check ui-igcheckbox-small-on"></span></span></span>{{/if}}';
                    }

                    //Work item #2964 -- right align number columns
                    if (columnAttributes.dataType == "number" && !columnAttributes.template) {
                        columnAttributes.template = '<div style="text-align:right">${' + key + '}</div>';
                    }

                    if (columnAttributes.dataType == "date" && !columnAttributes.format) {
                        columnAttributes.format = "MM/dd/yyyy";
                    }

                    features.hiding.columnSettings.push({ columnKey: key, hidden: hidden, allowHiding: hidden });

                    if (columnAttributes.update) {
                        features.update.columnSettings.push({ columnKey: key, editorOptions: { required: fieldRequired} });
                    }
                    else {
                        features.update.columnSettings.push({ columnKey: key, editorOptions: { disabled: true} });
                    }

                    if (columnAttributes.tooltips) features.tooltips.columnSettings.push({ columnKey: key, allowTooltips: true });


                    gridSettings.columns[index] = {
                        dataType: columnAttributes.dataType,
                        format: columnAttributes.format,
                        formatter: formatter,
                        headerText: headerText,
                        hidden: columnAttributes.display === "false" ? true : false,
                        key: key,
                        width: columnAttributes.width,
                        template: columnAttributes.template,
                        unbound: columnAttributes.unbound === "true" ? true : false
                    };

                    if (!columnAttributes.template) {

                        if (columnAttributes.tooltipUrl) {
                            ajaxTooltips.push({ columnKey: key,
                                url: columnAttributes.tooltipUrl,
                                width: columnAttributes.tooltipWidth,
                                tooltipColKey: columnAttributes.tooltipColKey
                            });
                        }


                        if (columnAttributes.colUrl) {
                            columnlinks.push({ columnKey: key,
                                url: columnAttributes.colUrl,
                                isExternalLink: columnAttributes.isExternalLink,
                                displayText: columnAttributes.displayText,
                                isPopupWindow: columnAttributes.isPopupWindow,
                                popupWindowTitle: columnAttributes.popupWindowTitle
                            });
                        }
                    }

                    features.sorting.columnSettings.push({ columnKey: key, allowSorting: sort });
                    features.filtering.columnSettings.push({ columnKey: key, allowFiltering: filter });


                });

                if (gridAttributes.source) {

                    gridSettings.dataSource = new $.ig.RemoteDataSource({
                        dataSource: gridAttributes.source,
                        responseDataKey: "d.results",
                        responseDataType: "jsonp",
                        responseTotalRecCountKey: "d.__count"
                    });

                }

                $table.igGrid(gridSettings);
            });
        },


        _dataBinding: function (source, schema) {
            source.schema(schema, "json");
        },

        _renderRows: function (grid, callback) {

            if (callback) {

                var fn = window[callback];

                if (typeof fn === 'function') {

                    var gridRows = grid.rows();
                    var dataRows = grid.dataSource.dataView();
                    var rowCount = gridRows.length;

                    for (var row = 0; row < rowCount; row++) {
                        fn(gridRows[row], dataRows[row]);
                    }
                }
            }

        },

        _renderTooltips: function (grid, tooltipSettings) {

            for (var tooltipIndex = 0; tooltipIndex < tooltipSettings.length; tooltipIndex++) {

                var tooltip = tooltipSettings[tooltipIndex];
                var c = this._getColumnDetail(grid, tooltipSettings, tooltipIndex);

                if (!c.column.hidden) {

                    var rows = grid.rows();
                    var rowCount = rows.length;

                    for (var rowIndex = 0; rowIndex < rowCount; rowIndex++) {

                        var cell = rows[rowIndex].cells[c.columnIndex];
                        var id = $(cell).html();

                        if (tooltip.tooltipColKey) {
                            id = grid.dataSource.dataView()[rowIndex][tooltip.tooltipColKey];
                        }

                        if (id && id != " " && id != "&nbsp;") {

                            var tooltipUrl = $.trim(tooltip.url.replace(regexes.queryRegex, "/" + id + "$1"));
                            $(cell).admedTooltip({ url: tooltipUrl, width: tooltip.width });
                        }

                    }
                }
            }
        },

        _renderLinks: function (grid, colLinks) {

            for (var colLinksIndex = 0; colLinksIndex < colLinks.length; colLinksIndex++) {

                var c = this._getColumnDetail(grid, colLinks, colLinksIndex);
                var colLink = colLinks[colLinksIndex];

                if (!c.column.hidden) {

                    var rows = grid.rows();

                    for (var rowIndex = 0; rowIndex < rows.length; rowIndex++) {
                        var cell = rows[rowIndex].cells[c.columnIndex];
                        var id = $(cell).html();
                        var displayText = $(cell).html();

                        if (colLink.displayText) {
                            displayText = colLink.displayText;
                        }

                        if (id && id != " ") {

                            if (colLink.isExternalLink) {
                                var colUrl = colLink.url + "?" + colLink.columnKey + "=" + id;
                                $(cell).html("<a href='" + colUrl + "' target='_blank' >" + displayText + "</a>");
                            }
                            else {

                                var colUrl = colLink.url.replace(regexes.queryRegex, "/" + id + "$1");

                                if (colLink.isPopupWindow) {
                                    $(cell).html("<a href='#' data-popupwindow='true' data-popupwindow-title = '" + colLink.popupWindowTitle + "' data-popupwindow-url='" + colUrl + "' >" + displayText + "</a>");

                                }
                                else {
                                    $(cell).html("<a href='" + colUrl + "'>" + displayText + "</a>");
                                }
                            }
                        }
                    }
                }
            }

        },

        _getColumnDetail: function (grid, colAttrs, currentIndex) {

            var column;
            var colAttr = colAttrs[currentIndex];
            var columnIndex = currentIndex;

            for (var index = currentIndex; index < grid.options.columns.length; index++) {
                column = grid.options.columns[index];

                if (column.key == colAttr.columnKey) {
                    break;
                }
                else if (!column.hidden) {
                    columnIndex++;
                }
            }

            return { column: column, columnIndex: columnIndex };
        },

        _rowSelectionChanged: function (evt, ui, features) {
            // Hide the row selection icon (the sideways triangle).
            ui.row.element.find(".ui-icon-triangle-1-e").hide();

            if (typeof window[features.selectionChanged] === "function") {
                window[features.selectionChanged](evt, ui);
            }
        }
    };

    $.fn.admedGrid = function (method) {

        if (typeof method === 'object' || !method) {
            return methods.admedGrid.apply(this, arguments);
        }
        else if (method.charAt(0) != "_" && methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        }
        else {
            $.error('Method ' + method + ' does not exist on jQuery.admedGrid');
        }
    };

})(jQuery);



